package com.dreamcodex.ti.component;

import com.dreamcodex.ti.Magellan;
import com.dreamcodex.ti.iface.IconProvider;
import com.dreamcodex.ti.util.IconChar;
import com.dreamcodex.ti.util.TIGlobals;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

public class MagellanExportDialog extends JDialog implements PropertyChangeListener, ActionListener {
    public static int TYPE_BASIC = 0;
    public static int TYPE_ASM = 1;
    public static int TYPE_BINARY = 2;
    public static int TYPE_XBSCRMER = 3;
    public static int TYPE_SCROLL = 4;

    private String OK_TEXT = "Export";
    private String CANCEL_TEXT = "Cancel";

    private JComboBox jcmbStartChar;
    private JComboBox jcmbEndChar;
    private JTextField jtxtCodeLineStart;
    private JTextField jtxtCharLineStart;
    private JTextField jtxtMapLineStart;
    private JTextField jtxtLineInterval;
    private JCheckBox jchkIncludeColorsets;
    private JCheckBox jchkIncludeChardata;
    private JCheckBox jchkIncludeSpritedata;
    private JCheckBox jchkIncludeComments;
    private JCheckBox jchkIncludeCharNumbers;
    private JCheckBox jchkCurrentMapOnly;
    private JPanel orientationRadioButtonPanel;
    private JRadioButton verticalButton;
    private JRadioButton horizontalButton;
    private JCheckBox jchkWrap;
    private JComboBox frameComboBox;
    private JComboBox compressComboBox;
    private boolean clickedOkay = false;
    private int minChar = 0;
    private int maxChar = 0;

    public MagellanExportDialog(int type, JFrame parent, IconProvider ip, boolean setCommentsOn, int startChar, int endChar, int minc, int maxc, boolean currentMapOnly, boolean includeCharNumbers) {
        super(parent, "Export Settings", true);
        minChar = minc;
        maxChar = maxc;
        if (type != TYPE_XBSCRMER) {
            jtxtCodeLineStart = new JTextField("100");
            jtxtCharLineStart = new JTextField("500");
        }
        else {
            jtxtCodeLineStart = new JTextField("30000");
            jtxtCharLineStart = new JTextField("28");
        }
        jtxtMapLineStart = new JTextField("900");
        jtxtLineInterval = new JTextField("10");
        jchkIncludeColorsets = new JCheckBox("Include Colorsets", true);
        jchkIncludeChardata = new JCheckBox("Include Character Data", true);
        jchkIncludeSpritedata = new JCheckBox("Include Sprite Data", false);
        jchkIncludeSpritedata.setEnabled(false);
        jchkIncludeComments = new JCheckBox("Include Comments", setCommentsOn);
        jchkIncludeCharNumbers = new JCheckBox("Include Character Numbers", includeCharNumbers);
        jchkCurrentMapOnly = new JCheckBox("Current Map Only", currentMapOnly);

        verticalButton = new JRadioButton("Bottom to Top", true);
        verticalButton.addActionListener(this);
        horizontalButton = new JRadioButton("Left to Right");
        horizontalButton.addActionListener(this);
        ButtonGroup radioButtonGroup = new ButtonGroup();
        radioButtonGroup.add(verticalButton);
        radioButtonGroup.add(horizontalButton);
        orientationRadioButtonPanel = new JPanel();
        orientationRadioButtonPanel.add(verticalButton);
        orientationRadioButtonPanel.add(horizontalButton);
        if (Magellan.ISOMETRIC) {
            JRadioButton isometricButton = new JRadioButton("Isometric", true);
            isometricButton.addActionListener(this);
            radioButtonGroup.add(isometricButton);
            orientationRadioButtonPanel.add(isometricButton);
        }
        add(orientationRadioButtonPanel, BorderLayout.NORTH);
        jchkWrap = new JCheckBox("Wrap Edges");
        compressComboBox = new JComboBox(new String[] {"No compression", "RLE Compress Maps (bytes)", "RLE Compress Maps (words)"});
        frameComboBox = new JComboBox(new String[] {"0", "2", "4", "8", "2-character Strips"});

        jcmbStartChar = new JComboBox();
        jcmbStartChar.setRenderer(new CharListCellRenderer());
        jcmbEndChar = new JComboBox();
        jcmbEndChar.setRenderer(new CharListCellRenderer());
        for (int i = minChar; i <= maxChar; i++) {
            Icon icon = ip.getIconForObject(i);
            int chardex = i - TIGlobals.CHARMAPSTART;
            IconChar iconChar = new IconChar(icon, (icon == null && chardex >= 0 && chardex < TIGlobals.CHARMAP.length ? TIGlobals.CHARMAP[chardex] + " " : "") + Integer.toString(i));
            jcmbStartChar.addItem(iconChar);
            jcmbEndChar.addItem(iconChar);
        }
        jcmbStartChar.setSelectedIndex(0);
        jcmbEndChar.setSelectedIndex(jcmbEndChar.getItemCount() - 1);

        Object[] objForm = new Object[15];
        int objCount = 0;
        if (type != TYPE_XBSCRMER && type != TYPE_SCROLL) {
            objForm[objCount++] = new JLabel("Character Range");
            objForm[objCount++] = new JLabel("From Char #");
            objForm[objCount++] = jcmbStartChar;
            objForm[objCount++] = new JLabel("To Char #");
            objForm[objCount++] = jcmbEndChar;
        }
        if (type == TYPE_BASIC) {
            objForm[objCount++] = new JLabel("Code Line Number Start");
            objForm[objCount++] = jtxtCodeLineStart;
            objForm[objCount++] = new JLabel("Character Data Line Number Start");
            objForm[objCount++] = jtxtCharLineStart;
            objForm[objCount++] = new JLabel("Map Data Line Number Start");
            objForm[objCount++] = jtxtMapLineStart;
            objForm[objCount++] = new JLabel("Line Number Interval");
            objForm[objCount++] = jtxtLineInterval;
        }
        if (type == TYPE_BINARY) {
            objForm[objCount++] = jchkIncludeColorsets;
            objForm[objCount++] = jchkIncludeChardata;
            objForm[objCount++] = jchkIncludeSpritedata;
        }
        if (type == TYPE_XBSCRMER) {
            objForm[objCount++] = new JLabel("Code Line Number Start (0-32710)");
            objForm[objCount++] = jtxtCodeLineStart;
            objForm[objCount++] = new JLabel("Display width (28 or 32)");
            objForm[objCount++] = jtxtCharLineStart;
        }
        if (type == TYPE_SCROLL) {
            objForm[objCount++] = orientationRadioButtonPanel;
            objForm[objCount++] = jchkWrap;
            objForm[objCount++] = compressComboBox;
            objForm[objCount++] = new JLabel("Generate Scrolled Character Frames");
            objForm[objCount++] = frameComboBox;
        }
        if (type != TYPE_XBSCRMER) {
            objForm[objCount++] = jchkCurrentMapOnly;
        }
        if (type == TYPE_BASIC || type == TYPE_ASM || type == TYPE_SCROLL) {
            objForm[objCount++] = jchkIncludeComments;
        }
        if (type == TYPE_ASM || type == TYPE_SCROLL) {
            objForm[objCount++] = jchkIncludeCharNumbers;
        }
        Object[] objButtons = {OK_TEXT, CANCEL_TEXT};

        JOptionPane joptMain = new JOptionPane(objForm, JOptionPane.QUESTION_MESSAGE, JOptionPane.OK_CANCEL_OPTION, null, objButtons, objButtons[0]);
        joptMain.addPropertyChangeListener(this);
        this.setContentPane(joptMain);

        this.setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
        this.pack();
        this.setLocationRelativeTo(this);
        this.setVisible(true);
    }

    public MagellanExportDialog(int type, JFrame parent, IconProvider ip, boolean setCommentsOn, int startChar, int endChar, int minc, int maxc, boolean currentMapOnly) {
        this(type, parent, ip, setCommentsOn, startChar, endChar, minc, maxc, currentMapOnly, false);
    }

    public int getStartChar() {
        return jcmbStartChar.getSelectedIndex() + minChar;
    }

    public int getEndChar() {
        return jcmbEndChar.getSelectedIndex() + minChar;
    }

    public int getCodeLineStart() {
        return Integer.parseInt(jtxtCodeLineStart.getText());
    }

    public int getCharLineStart() {
        return Integer.parseInt(jtxtCharLineStart.getText());
    }

    public int getMapLineStart() {
        return Integer.parseInt(jtxtMapLineStart.getText());
    }

    public int getLineInterval() {
        return Integer.parseInt(jtxtLineInterval.getText());
    }

    public boolean includeColorsets() {
        return jchkIncludeColorsets.isSelected();
    }

    public boolean includeChardata() {
        return jchkIncludeChardata.isSelected();
    }

    public boolean includeSpritedata() {
        return jchkIncludeSpritedata.isSelected();
    }

    public boolean includeComments() {
        return jchkIncludeComments.isSelected();
    }

    public boolean includeCharNumbers() {
        return jchkIncludeCharNumbers.isSelected();
    }

    public boolean currentMapOnly() {
        return jchkCurrentMapOnly.isSelected();
    }

    public boolean isVertical() {
        return verticalButton.isSelected();
    }

    public int getScrollOrientation() {
        return verticalButton.isSelected() ? Magellan.SCROLL_ORIENTATION_VERTICAL : (horizontalButton.isSelected() ? Magellan.SCROLL_ORIENTATION_HORIZONTAL : Magellan.SCROLL_ORIENTATION_ISOMETRIC);
    }

    public boolean isWrap() {
        return jchkWrap.isSelected();
    }

    public boolean isCompress() {
        return compressComboBox.getSelectedIndex() != 0;
    }

    public boolean isCompressWords() {
        return compressComboBox.getSelectedIndex() == 2;
    }

    public int getFrames() {
        String frames = (String) frameComboBox.getSelectedItem();
        try {
            return Integer.parseInt(frames);
        } catch (NumberFormatException e) {
            return -1;
        }
    }

    public boolean isOkay() {
        return clickedOkay;
    }

    /* PropertyChangeListener method */
    public void propertyChange(PropertyChangeEvent pce) {
        if (pce != null && pce.getNewValue() != null) {
            if (pce.getNewValue().equals(OK_TEXT)) {
                clickedOkay = true;
                this.setVisible(false);
            }
            else if (pce.getNewValue().equals(CANCEL_TEXT)) {
                clickedOkay = false;
                this.setVisible(false);
            }
        }
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == verticalButton) {
            frameComboBox.addItem("2-character Strips");
        }
        if (e.getSource() == horizontalButton) {
            frameComboBox.removeItemAt(4);
        }
    }
}

